pub(super) static VERSION: &str = r"2021a";

pub(super) static WINDOWS_TO_IANA: &[(&str, &str)] = &[
    (r"Afghanistan Standard Time", r"Asia/Kabul"),
    (r"Alaskan Standard Time", r"America/Anchorage"),
    (r"Aleutian Standard Time", r"America/Adak"),
    (r"Altai Standard Time", r"Asia/Barnaul"),
    (r"Arab Standard Time", r"Asia/Riyadh"),
    (r"Arabian Standard Time", r"Asia/Dubai"),
    (r"Arabic Standard Time", r"Asia/Baghdad"),
    (r"Argentina Standard Time", r"America/Buenos_Aires"),
    (r"Astrakhan Standard Time", r"Europe/Astrakhan"),
    (r"Atlantic Standard Time", r"America/Halifax"),
    (r"AUS Central Standard Time", r"Australia/Darwin"),
    (r"Aus Central W. Standard Time", r"Australia/Eucla"),
    (r"AUS Eastern Standard Time", r"Australia/Sydney"),
    (r"Azerbaijan Standard Time", r"Asia/Baku"),
    (r"Azores Standard Time", r"Atlantic/Azores"),
    (r"Bahia Standard Time", r"America/Bahia"),
    (r"Bangladesh Standard Time", r"Asia/Dhaka"),
    (r"Belarus Standard Time", r"Europe/Minsk"),
    (r"Bougainville Standard Time", r"Pacific/Bougainville"),
    (r"Canada Central Standard Time", r"America/Regina"),
    (r"Cape Verde Standard Time", r"Atlantic/Cape_Verde"),
    (r"Caucasus Standard Time", r"Asia/Yerevan"),
    (r"Cen. Australia Standard Time", r"Australia/Adelaide"),
    (r"Central America Standard Time", r"America/Guatemala"),
    (r"Central Asia Standard Time", r"Asia/Bishkek"),
    (r"Central Brazilian Standard Time", r"America/Cuiaba"),
    (r"Central Europe Standard Time", r"Europe/Budapest"),
    (r"Central European Standard Time", r"Europe/Warsaw"),
    (r"Central Pacific Standard Time", r"Pacific/Guadalcanal"),
    (r"Central Standard Time", r"America/Chicago"),
    (r"Central Standard Time (Mexico)", r"America/Mexico_City"),
    (r"Chatham Islands Standard Time", r"Pacific/Chatham"),
    (r"China Standard Time", r"Asia/Shanghai"),
    (r"Cuba Standard Time", r"America/Havana"),
    (r"Dateline Standard Time", r"Etc/GMT+12"),
    (r"E. Africa Standard Time", r"Africa/Nairobi"),
    (r"E. Australia Standard Time", r"Australia/Brisbane"),
    (r"E. Europe Standard Time", r"Europe/Chisinau"),
    (r"E. South America Standard Time", r"America/Sao_Paulo"),
    (r"Easter Island Standard Time", r"Pacific/Easter"),
    (r"Eastern Standard Time", r"America/New_York"),
    (r"Eastern Standard Time (Mexico)", r"America/Cancun"),
    (r"Egypt Standard Time", r"Africa/Cairo"),
    (r"Ekaterinburg Standard Time", r"Asia/Yekaterinburg"),
    (r"Fiji Standard Time", r"Pacific/Fiji"),
    (r"FLE Standard Time", r"Europe/Kiev"),
    (r"Georgian Standard Time", r"Asia/Tbilisi"),
    (r"GMT Standard Time", r"Europe/London"),
    (r"Greenland Standard Time", r"America/Godthab"),
    (r"Greenwich Standard Time", r"Atlantic/Reykjavik"),
    (r"GTB Standard Time", r"Europe/Bucharest"),
    (r"Haiti Standard Time", r"America/Port-au-Prince"),
    (r"Hawaiian Standard Time", r"Pacific/Honolulu"),
    (r"India Standard Time", r"Asia/Calcutta"),
    (r"Iran Standard Time", r"Asia/Tehran"),
    (r"Israel Standard Time", r"Asia/Jerusalem"),
    (r"Jordan Standard Time", r"Asia/Amman"),
    (r"Kaliningrad Standard Time", r"Europe/Kaliningrad"),
    (r"Korea Standard Time", r"Asia/Seoul"),
    (r"Libya Standard Time", r"Africa/Tripoli"),
    (r"Line Islands Standard Time", r"Pacific/Kiritimati"),
    (r"Lord Howe Standard Time", r"Australia/Lord_Howe"),
    (r"Magadan Standard Time", r"Asia/Magadan"),
    (r"Magallanes Standard Time", r"America/Punta_Arenas"),
    (r"Marquesas Standard Time", r"Pacific/Marquesas"),
    (r"Mauritius Standard Time", r"Indian/Mauritius"),
    (r"Middle East Standard Time", r"Asia/Beirut"),
    (r"Montevideo Standard Time", r"America/Montevideo"),
    (r"Morocco Standard Time", r"Africa/Casablanca"),
    (r"Mountain Standard Time", r"America/Denver"),
    (r"Mountain Standard Time (Mexico)", r"America/Mazatlan"),
    (r"Myanmar Standard Time", r"Asia/Rangoon"),
    (r"N. Central Asia Standard Time", r"Asia/Novosibirsk"),
    (r"Namibia Standard Time", r"Africa/Windhoek"),
    (r"Nepal Standard Time", r"Asia/Katmandu"),
    (r"New Zealand Standard Time", r"Pacific/Auckland"),
    (r"Newfoundland Standard Time", r"America/St_Johns"),
    (r"Norfolk Standard Time", r"Pacific/Norfolk"),
    (r"North Asia East Standard Time", r"Asia/Irkutsk"),
    (r"North Asia Standard Time", r"Asia/Krasnoyarsk"),
    (r"North Korea Standard Time", r"Asia/Pyongyang"),
    (r"Omsk Standard Time", r"Asia/Omsk"),
    (r"Pacific SA Standard Time", r"America/Santiago"),
    (r"Pacific Standard Time", r"America/Los_Angeles"),
    (r"Pacific Standard Time (Mexico)", r"America/Tijuana"),
    (r"Pakistan Standard Time", r"Asia/Karachi"),
    (r"Paraguay Standard Time", r"America/Asuncion"),
    (r"Qyzylorda Standard Time", r"Asia/Qyzylorda"),
    (r"Romance Standard Time", r"Europe/Paris"),
    (r"Russia Time Zone 10", r"Asia/Srednekolymsk"),
    (r"Russia Time Zone 11", r"Asia/Kamchatka"),
    (r"Russia Time Zone 3", r"Europe/Samara"),
    (r"Russian Standard Time", r"Europe/Moscow"),
    (r"SA Eastern Standard Time", r"America/Cayenne"),
    (r"SA Pacific Standard Time", r"America/Bogota"),
    (r"SA Western Standard Time", r"America/La_Paz"),
    (r"Saint Pierre Standard Time", r"America/Miquelon"),
    (r"Sakhalin Standard Time", r"Asia/Sakhalin"),
    (r"Samoa Standard Time", r"Pacific/Apia"),
    (r"Sao Tome Standard Time", r"Africa/Sao_Tome"),
    (r"Saratov Standard Time", r"Europe/Saratov"),
    (r"SE Asia Standard Time", r"Asia/Bangkok"),
    (r"Singapore Standard Time", r"Asia/Singapore"),
    (r"South Africa Standard Time", r"Africa/Johannesburg"),
    (r"South Sudan Standard Time", r"Africa/Juba"),
    (r"Sri Lanka Standard Time", r"Asia/Colombo"),
    (r"Sudan Standard Time", r"Africa/Khartoum"),
    (r"Syria Standard Time", r"Asia/Damascus"),
    (r"Taipei Standard Time", r"Asia/Taipei"),
    (r"Tasmania Standard Time", r"Australia/Hobart"),
    (r"Tocantins Standard Time", r"America/Araguaina"),
    (r"Tokyo Standard Time", r"Asia/Tokyo"),
    (r"Tomsk Standard Time", r"Asia/Tomsk"),
    (r"Tonga Standard Time", r"Pacific/Tongatapu"),
    (r"Transbaikal Standard Time", r"Asia/Chita"),
    (r"Turkey Standard Time", r"Europe/Istanbul"),
    (r"Turks And Caicos Standard Time", r"America/Grand_Turk"),
    (r"Ulaanbaatar Standard Time", r"Asia/Ulaanbaatar"),
    (r"US Eastern Standard Time", r"America/Indianapolis"),
    (r"US Mountain Standard Time", r"America/Phoenix"),
    (r"UTC", r"Etc/UTC"),
    (r"UTC+12", r"Etc/GMT-12"),
    (r"UTC+13", r"Etc/GMT-13"),
    (r"UTC-02", r"Etc/GMT+2"),
    (r"UTC-08", r"Etc/GMT+8"),
    (r"UTC-09", r"Etc/GMT+9"),
    (r"UTC-11", r"Etc/GMT+11"),
    (r"Venezuela Standard Time", r"America/Caracas"),
    (r"Vladivostok Standard Time", r"Asia/Vladivostok"),
    (r"Volgograd Standard Time", r"Europe/Volgograd"),
    (r"W. Australia Standard Time", r"Australia/Perth"),
    (r"W. Central Africa Standard Time", r"Africa/Lagos"),
    (r"W. Europe Standard Time", r"Europe/Berlin"),
    (r"W. Mongolia Standard Time", r"Asia/Hovd"),
    (r"West Asia Standard Time", r"Asia/Tashkent"),
    (r"West Bank Standard Time", r"Asia/Hebron"),
    (r"West Pacific Standard Time", r"Pacific/Port_Moresby"),
    (r"Yakutsk Standard Time", r"Asia/Yakutsk"),
    (r"Yukon Standard Time", r"America/Whitehorse"),
];
